﻿using AnalyzeTask.Model;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using AnalyzeTask.Tool.Dapper;
using Serilog;
using AnalyzeTask.Tool;
using Newtonsoft.Json;

namespace AnalyzeTask.DataSource
{
    /// <summary>
    /// 数据操作
    /// </summary>
    public class DataManipulation : IDisposable
    {
        private ILogger log = Serilogs.logger;
        /// <summary>
        /// 获取某一项指标的数据
        /// </summary>
        /// <param name="needSynData"></param>
        /// <param name="oracleSourceDB"></param>
        /// <param name="Tag">某一项指标，如：血常规、血生化</param>
        public DataTable GetOneInDicatorsByTag(string Tag)
        {
            DataTable newTable = new DataTable();
            //找出当前需要的指标
            DataTable TagList = NeedSynData.GettableConfigs(Tag);
            StringBuilder InFo = new StringBuilder();
            if (TagList.Rows.Count > 0)
            {
                InFo.Append("(");
                for (int i = 0; i < TagList.Rows.Count; i++)
                {
                    if (i != TagList.Rows.Count - 1)
                    {
                        InFo.Append("'");
                        InFo.Append(TagList.Rows[i][0]);
                        InFo.Append("',");
                    }
                    else
                    {
                        InFo.Append("'");
                        InFo.Append(TagList.Rows[i][0]);
                        InFo.Append("'");
                    }
                }
                InFo.Append(")");
                //在我们数据库中的字段就获取，没有就不获取
                //这里修改为从数据库中查找。
                DataTable DataSource = OracleSourceDB.GetResult(InFo.ToString());
                if (DataSource.Rows.Count > 0)
                {
                    newTable = DataSource;
                }
            }
            log.Debug($"{Tag}结果指标数据有{newTable.Rows.Count}条");
            return newTable;
        }

        /// <summary>
        /// 获取指标的配置数据
        /// </summary>
        /// <param name="Tag"></param>
        /// <returns></returns>
        public List<TableConfig> GetTagTableConfig(string Tag)
        {
            var TagTableConfig = NeedSynData.GetAlltableConfigs(Tag);
            return TagTableConfig;
        }

        /// <summary>
        /// 将数据插入到数据库
        /// </summary>
        /// <param name="configs">表配置</param>
        /// <param name="Rows">数据源</param>
        /// <param name="keyValuePairs">双方表的字段对应</param>
        /// <param name="action">回调</param>
        public void InsertIntoMSSQL(List<TableConfig> configs, DataTable Table, Action<Task> action)
        {
            Task task = Task.Run(() =>
            {
                string Rval = "";
                string RConlum = "";
                string TableName = string.Empty;
                try
                {
                    //判断是否重复的数据
                    List<string> Conlums = new List<string>();
                    //获取要插入的表
                    TableName = configs.Distinct().Select(n => n.DYBD).FirstOrDefault().Trim();
                    //获取要插入的字段
                    StringBuilder Conlum = new StringBuilder();
                    StringBuilder Val = new StringBuilder();
                    string ConlumName = string.Empty;
                    string Value = string.Empty;
                    List<CheckRepeatData> repeatDatas = new List<CheckRepeatData>();
                    //寻找当前人的数据
                    NeedSynData.pregnancies.ForEach(n =>
                    {
                        DataRow[] Rows = Table.Select($"INPATIENT_ID='{n.AttendanceCard}'").Distinct().ToArray();//当前人所有的信息
                        if (Rows.Length > 0)
                        {
                            DataTable dt = Rows.CopyToDataTable();
                            DataView dv = dt.DefaultView;
                            DataTable Date = dv.ToTable(true, "INSERT_TIME");
                            for (int k = 0; k < Date.Rows.Count; k++)
                            {
                                DataRow[] rows = dt.Select($"INSERT_TIME='{Date.Rows[k]["INSERT_TIME"]}'").Distinct().ToArray();
                                if (rows.Length > 0)
                                {
                                    DataTable dts = rows.CopyToDataTable();
                                    DataView dvs = dts.DefaultView;
                                    DataTable Datas = dvs.ToTable(true, "INSERT_TIME", "INPATIENT_ID", "TEST_ITEM_ID", "QUANTITATIVE_RESULT");
                                    rows = Datas.Select("1=1");
                                    Conlum.Append($"INSERT INTO {TableName}(");
                                    Val.Append($" VALUES (");
                                    for (int i = 0; i < rows.Length; i++)
                                    {
                                        if (i == 0)
                                        {
                                            Conlum.Append($"JCRQ,ZKKH,{TableName}OID,");
                                            Value = rows[i]["QUANTITATIVE_RESULT"].ToString();
                                            Val.Append($"'{rows[i]["INSERT_TIME"].ToString()}','{ StringHelp.StringRemoveLeft('0', rows[i]["INPATIENT_ID"].ToString()) }','{Guid.NewGuid()}',");
                                        }
                                        ConlumName = configs.FindAll(m => m.ZDID == rows[i]["TEST_ITEM_ID"].ToString()).Select(m => m.ZDMC).FirstOrDefault().Trim();//找出配置表中这条数据应该放在哪一列
                                        if (string.IsNullOrEmpty(Conlums.FirstOrDefault(f => f == ConlumName)))
                                        {
                                            if (!string.IsNullOrEmpty(ConlumName))
                                            {
                                                Conlum.Append($"{ConlumName},");
                                                Value = rows[i]["QUANTITATIVE_RESULT"].ToString();
                                                Val.Append($"'{Value}',");
                                                Conlums.Add(ConlumName);
                                            }
                                        }
                                    }
                                    Rval = Val.ToString().Substring(0, Val.ToString().Length - 1);
                                    RConlum = Conlum.ToString().Substring(0, Conlum.ToString().Length - 1);
                                    RConlum += $")";
                                    Rval += $")";
                                    DateTime time = new DateTime();
                                    DateTime.TryParse(rows[0]["INSERT_TIME"].ToString(), out time);
                                    //log.Debug($"插入语句{Conlum.ToString() + Val.ToString()}\r\n");
                                    repeatDatas.Add(new CheckRepeatData { SQL = RConlum + Rval, Card = StringHelp.StringRemoveLeft('0', n.AttendanceCard), Time = time.ToString("yyyy/MM/DD HH:mm:ss") });
                                    Conlums.Clear();
                                    Conlum.Clear();
                                    RConlum = "";
                                    Rval = "";
                                    Val.Clear();
                                }
                            }
                            Date.Clear();
                        }
                    });
                    //对重复的数据进行过滤
                    //log.Debug($"命令行中数据条数{repeatDatas.Count}");
                    //repeatDatas = CheckRepeatDatas(repeatDatas, TableName);
                    CommandCollection command = new CommandCollection();
                    repeatDatas.ForEach(n =>
                    {
                        command.Add(new Command(n.SQL));
                    });
                    //插入数据库
                    NeedSynData.dbContext.ExecuteTransaction(command);
                }
                catch (Exception ex)
                {
                    log.Debug(ex, $"{TableName}----InsertIntoMSSQL出错" + $"插入语句{RConlum.ToString() + Rval.ToString()}\r\n");
                }

            });
            action(task);
        }

        /// <summary>
        /// 检查已经有的数据
        /// </summary>
        /// <param name="repeatDatas"></param>
        /// <param name="TableName"></param>
        /// <returns></returns>
        private List<CheckRepeatData> CheckRepeatDatas(List<CheckRepeatData> repeatDatas, string TableName)
        {
            log.Debug("开始检查数据");
            List<CheckRepeatData> data = new List<CheckRepeatData>();
            DataTable HasData = NeedSynData.GetHasData(TableName);
            foreach (var item in repeatDatas)
            {
                DataRow[] row = HasData.Select($"ZKKH='{item.Card}' AND JCRQ='{item.Time}'");
                if (row.Length > 0)
                {
                    repeatDatas.Remove(item);
                }
            }
            return repeatDatas;
        }

        public void Dispose()
        {
            log.Debug("DataManipulation资源被释放");
            GC.SuppressFinalize(this);
        }
    }
}
