﻿using AnalyzeTask.Model;
using AnalyzeTask.Tool;
using AnalyzeTask.Tool.Dapper;
using Serilog;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AnalyzeTask.DataSource
{
    /// <summary>
    /// 获取需要同步的数据
    /// </summary>
    public class NeedSynData : IDisposable
    {
        /// <summary>
        /// 需要同步的数据集合
        /// </summary>
        public static List<string> DateTimeList = new List<string>();
        public static List<Pregnancy> pregnancies = new List<Pregnancy>();
        public static List<TableConfig> tableConfigs = new List<TableConfig>();
        public static DbContext dbContext = new DbContext().ChangeContext(DBType.SQLSERVER);
        private static ILogger log = Serilogs.logger;

        public void GetNeedSynData()
        {
            //从任务表中获取需要同步的日期
            DateTimeList = dbContext.Query<string>("SELECT ImportDate FROM AnalysisofLaboratoryIndicator WHERE status=2").ToList();
            //获取需要同步的成员数据
            pregnancies = dbContext.Query<Pregnancy>("SELECT * FROM dbo.Pregnancy WHERE ImportDate IN (SELECT ImportDate FROM AnalysisofLaboratoryIndicator WHERE status=2)").ToList();
            pregnancies.ForEach(n => { n.AttendanceCard = StringHelp.StringPadLeft('0', 12, n.AttendanceCard); });//将就诊卡号补0,12位
            //获取需要获取的数据表单配置
            tableConfigs = dbContext.Query<TableConfig>("SELECT * FROM TableConfig").ToList();
            //清空表数据
            DelTableData();
        }

        public static DataTable GettableConfigs(string TaskName)
        {
            string sql = $"SELECT DISTINCT ZDID FROM dbo.TableConfig WHERE SSXM LIKE '%{TaskName.Trim()}%'";
            DataTable table = dbContext.ExecuteDataTable(sql, CommandType.Text);
            log.Debug($"当前项目：{TaskName}总共条数:{table.Rows.Count}");
            if (table.Rows.Count == 0)
            {
                log.Debug($"SELECT DISTINCT ZDID FROM dbo.TableConfig WHERE SSXM LIKE '%{TaskName.Trim()}%'");
            }
            return table;
        }

        /// <summary>
        /// 获取全部的配置数据
        /// </summary>
        /// <param name="TaskName"></param>
        /// <returns></returns>
        public static List<TableConfig> GetAlltableConfigs(string TaskName)
        {
            string sql = $"SELECT * FROM dbo.TableConfig WHERE SSXM LIKE '%{TaskName.Trim()}%'";
            List<TableConfig> table = dbContext.Query<TableConfig>(sql, CommandType.Text).ToList();
            log.Debug($"当前项目：{TaskName}总共条数:{table.Count}");
            if (table.Count == 0)
            {
                log.Debug($"SELECT DISTINCT ZDID FROM dbo.TableConfig WHERE SSXM LIKE '%{TaskName.Trim()}%'");
            }
            return table;
        }
        /// <summary>
        /// 获取某一张表中已经有的数据
        /// </summary>
        /// <param name="TableName"></param>
        /// <returns></returns>
        public static DataTable GetHasData(string TableName)
        {
            try
            {
                DataTable data = dbContext.ExecuteDataTable($"SELECT A.* FROM {TableName} A LEFT JOIN Pregnancy B ON A.ZKKH=B.AttendanceCard WHERE B.ImportDate IN(SELECT ImportDate FROM AnalysisofLaboratoryIndicator WHERE status = 2)", CommandType.Text);
                return data;
            }
            catch (Exception)
            {
                Log.Debug($"SELECT A.* FROM {TableName} A LEFT JOIN Pregnancy B ON A.ZKKH=B.AttendanceCard WHERE B.ImportDate IN(SELECT ImportDate FROM AnalysisofLaboratoryIndicator WHERE status = 2)");
            }
            return new DataTable();
        }

        /// <summary>
        /// 清空数据
        /// </summary>
        private void DelTableData()
        {
            CommandCollection command = new CommandCollection();
            command.Add(new Command("truncate table RoutineBlood"));
            command.Add(new Command("truncate table RoutineUrine"));
            command.Add(new Command("truncate table TheBloodBiochemistry"));
            command.Add(new Command("truncate table CoagulationFunction"));
            command.Add(new Command("truncate table OralGlucoseToleranceTest"));
            command.Add(new Command("truncate table GlycosylatedAlbumin"));
            command.Add(new Command("truncate table GlycatedHemoglobin"));
            command.Add(new Command("truncate table ThyroidFunction"));
            command.Add(new Command("truncate table ScreeningForInfectious"));
            command.Add(new Command("truncate table TORCH"));
            command.Add(new Command("truncate table ImmuneIndexesOfRheumatism"));
            command.Add(new Command("truncate table Complement"));
            command.Add(new Command("truncate table LupusAnticoagulants"));
            command.Add(new Command("truncate table anemia"));
            NeedSynData.dbContext.ExecuteTransaction(command);
            log.Debug("结果表数据已清除");
        }

        public void Dispose()
        {
            log.Debug("NeedSynData资源被释放");
            DateTimeList = null;
            pregnancies = null;
            tableConfigs = null;
            GC.SuppressFinalize(this);
        }
    }
}
