﻿using AnalyzeTask.Tool;
using AnalyzeTask.Tool.Dapper;
using Serilog;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AnalyzeTask.DataSource
{
    public class OracleSourceDB : IDisposable
    {
        private DbContext dbContext = new DbContext().ChangeContext(DBType.Oracle);
        private static DbContext dbContextResult = new DbContext().ChangeContext(DBType.OracleResult);
        private DbContext dbContextMSDB = new DbContext().ChangeContext(DBType.SQLSERVER);
        public static DataTable AllBRResult = new DataTable();
        protected static ILogger log = Serilogs.logger;
        #region 获取his系统数据
        /// <summary>
        /// 获取病人信息
        /// </summary>
        /// <param name="MZH"></param>
        /// <returns></returns>
        public virtual DataTable GetBRInfoByJZK(string JZK)
        {
            return dbContext.OracleExcuteDatatable($"SELECT  *  from  bdyy.d2101  WHERE d2101.jiu_zhen_ka='{JZK}'");
        }


        public static DataTable GetResult(string info)
        {
            return dbContextResult.ExecuteDataTable($"SELECT * FROM ResultTable WHERE TEST_ITEM_ID IN {info}", CommandType.Text);
        }
        /// <summary>
        /// 获取门诊病人信息
        /// </summary>
        /// <param name="JZK"></param>
        /// <returns></returns>
        public virtual DataTable GetBrInfoMZXX(string JZK)
        {
            return dbContext.OracleExcuteDatatable($"SELECT  *  from  bdyy.d2101  WHERE d2101.jiu_zhen_ka='{JZK}'");
        }
        /// <summary>
        /// 获取门诊检查信息
        /// </summary>
        /// <param name="JZK"></param>
        /// <returns></returns>
        public virtual DataTable GetCheckProjectByJZK(string JZK)
        {
            return dbContext.OracleExcuteDatatable($"select * from xhlis.his_inspection_sample where INPATIENT_ID='{JZK}' ORDER BY SAMPLING_TIME");
        }
        /// <summary>
        /// 根据就诊卡获取病历号
        /// </summary>
        /// <returns></returns>
        public virtual string GetBLHByJZK(string JZK)
        {
            DataTable data = dbContext.OracleExcuteDatatable($"SELECT MEN_ZHEN_BLH from  bdyy.d2101  WHERE d2101.jiu_zhen_ka='{JZK}'");
            if (data.Rows.Count > 0)
            {
                return data.Rows[0][0].ToString();
            }
            return "";
        }

        /// <summary>
        /// 根据病历号获取检查结果
        /// </summary>
        /// <returns></returns>
        public virtual DataTable GetAllCheckResultByBLH(string BLH)
        {
            DataTable data = new DataTable();
            if (!string.IsNullOrEmpty(BLH))
            {
                data = dbContext.OracleExcuteDatatable($"select distinct A.TEST_ITEM_ID,A.ENGLISH_NAME,A.CHINESE_NAME,A.QUANTITATIVE_RESULT,B.INPATIENT_ID,A.INSERT_TIME from  xhlis.his_inspection_result A left join xhlis.his_inspection_sample B on A.INSPECTION_ID=B.INSPECTION_ID where A.INSPECTION_ID in (select INSPECTION_ID from xhlis.his_inspection_sample where OUTPATIENT_ID = '{BLH}' )");
            }
            return data;
        }

        /// <summary>
        /// 获取所有的门诊检查结果
        /// </summary>
        /// <param name="JZK"></param>
        /// <returns></returns>
        public virtual DataTable GetAllCheckResultByJZK(string JZK)
        {
            return dbContext.OracleExcuteDatatable($"select TEST_ITEM_ID,ENGLISH_NAME,CHINESE_NAME,QUANTITATIVE_RESULT,INSERT_TIME from  xhlis.his_inspection_result where INSPECTION_ID in (select INSPECTION_ID from xhlis.his_inspection_sample where INPATIENT_ID='{JZK}')");
        }

        /// <summary>
        /// 采用循环遍历的方式
        /// </summary>
        /// <param name="JZK"></param>
        /// <returns></returns>
        public virtual void GetAllCheckResultByJZKOne(string[] JZK)
        {
            TruncateResult();
            for (int i = 0; i < JZK.Length; i++)
            {
                DataTable data = new DataTable();
                data = dbContext.OracleExcuteDatatable($"select distinct A.TEST_ITEM_ID,A.ENGLISH_NAME,A.CHINESE_NAME,A.QUANTITATIVE_RESULT,B.INPATIENT_ID,A.INSERT_TIME from  xhlis.his_inspection_result A left join xhlis.his_inspection_sample B on A.INSPECTION_ID=B.INSPECTION_ID where A.INSPECTION_ID in (select INSPECTION_ID from xhlis.his_inspection_sample where INPATIENT_ID = '{JZK[i]}')");
                //将数据放到数据库中，不再放到内存中进行处理。
                CommandCollection commands = new CommandCollection();
                for (int j = 0; j < data.Rows.Count; j++)
                {
                    commands.Add(new Command($"INSERT INTO ResultTable VALUES('{data.Rows[j]["TEST_ITEM_ID"]}','{data.Rows[j]["ENGLISH_NAME"]}','{data.Rows[j]["CHINESE_NAME"]}','{data.Rows[j]["QUANTITATIVE_RESULT"]}','{data.Rows[j]["INPATIENT_ID"]}','{data.Rows[j]["INSERT_TIME"]}')"));
                }
                //根据病历号获取检查结果(查找的数据和上面的是一样的)
                //string BLH = GetBLHByJZK(JZK[i]);
                //DataTable MZData = GetAllCheckResultByBLH(BLH);
                //for (int j = 0; j < MZData.Rows.Count; j++)
                //{
                //    commands.Add(new Command($"INSERT INTO ResultTable VALUES('{MZData.Rows[j]["TEST_ITEM_ID"]}','{MZData.Rows[j]["ENGLISH_NAME"]}','{MZData.Rows[j]["CHINESE_NAME"]}','{MZData.Rows[j]["QUANTITATIVE_RESULT"]}','{MZData.Rows[j]["INPATIENT_ID"]}','{MZData.Rows[j]["INSERT_TIME"]}')"));
                //}
                log.Debug($"当前加载{JZK[i]},条数{commands.GetCount()}");
                dbContextResult.ExecuteTransaction(commands);
            }
        }

        /// <summary>
        /// 获取所有的门诊检查结果
        /// </summary>
        /// <param name="JZK"></param>
        /// <returns></returns>
        [Obsolete]
        public virtual DataTable GetAllCheckResultByJZK(string[] JZK)
        {
            int MaxParam = 300;
            StringBuilder InJZK = new StringBuilder();
            try
            {
                if (JZK.Length == 0)
                {
                    return AllBRResult;
                }
                if (JZK.Length < MaxParam)
                {
                    InJZK.Append("(");
                    for (int i = 0; i < JZK.Length; i++)
                    {
                        InJZK.Append("'");
                        InJZK.Append(JZK[i]);
                        if (i != JZK.Length - 1)
                        {
                            InJZK.Append("',");
                        }
                        else
                        {
                            InJZK.Append("')");
                        }
                    }
                    AllBRResult = dbContext.OracleExcuteDatatable($"select distinct A.TEST_ITEM_ID,A.ENGLISH_NAME,A.CHINESE_NAME,A.QUANTITATIVE_RESULT,B.INPATIENT_ID,A.INSERT_TIME from  xhlis.his_inspection_result A left join xhlis.his_inspection_sample B on A.INSPECTION_ID=B.INSPECTION_ID where A.INSPECTION_ID in (select INSPECTION_ID from xhlis.his_inspection_sample where INPATIENT_ID in {InJZK.ToString()})");
                    log.Debug($"select distinct A.TEST_ITEM_ID,A.ENGLISH_NAME,A.CHINESE_NAME,A.QUANTITATIVE_RESULT,B.INPATIENT_ID,A.INSERT_TIME from  xhlis.his_inspection_result A left join xhlis.his_inspection_sample B on A.INSPECTION_ID=B.INSPECTION_ID where A.INSPECTION_ID in (select INSPECTION_ID from xhlis.his_inspection_sample where INPATIENT_ID in {InJZK.ToString()})");
                }
                else
                {
                    int len = JZK.Length;
                    List<DataTable> dataTables = new List<DataTable>();
                    do
                    {
                        InJZK.Clear();
                        InJZK.Append("(");
                        for (int i = JZK.Length - len; i < JZK.Length - len + MaxParam; i++)
                        {
                            InJZK.Append("'");
                            InJZK.Append(JZK[i]);
                            if (i != JZK.Length - len + MaxParam - 1)
                            {
                                InJZK.Append("',");
                            }
                            else
                            {
                                InJZK.Append("')");
                            }
                        }
                        len = len - MaxParam;
                        AllBRResult = dbContext.OracleExcuteDatatable($"select distinct A.TEST_ITEM_ID,A.ENGLISH_NAME,A.CHINESE_NAME,A.QUANTITATIVE_RESULT,B.INPATIENT_ID,A.INSERT_TIME from  xhlis.his_inspection_result A left join xhlis.his_inspection_sample B on A.INSPECTION_ID=B.INSPECTION_ID where A.INSPECTION_ID in (select INSPECTION_ID from xhlis.his_inspection_sample where INPATIENT_ID in {InJZK.ToString()})");
                        log.Debug($"select distinct A.TEST_ITEM_ID,A.ENGLISH_NAME,A.CHINESE_NAME,A.QUANTITATIVE_RESULT,B.INPATIENT_ID,A.INSERT_TIME from  xhlis.his_inspection_result A left join xhlis.his_inspection_sample B on A.INSPECTION_ID=B.INSPECTION_ID where A.INSPECTION_ID in (select INSPECTION_ID from xhlis.his_inspection_sample where INPATIENT_ID in {InJZK.ToString()})");
                        dataTables.Add(AllBRResult);
                    } while (len % MaxParam != len);
                    InJZK.Clear();
                    InJZK.Append("(");
                    for (int i = JZK.Length - len; i < JZK.Length; i++)
                    {
                        InJZK.Append("'");
                        InJZK.Append(JZK[i]);
                        if (i != JZK.Length - 1)
                        {
                            InJZK.Append("',");
                        }
                        else
                        {
                            InJZK.Append("')");
                        }
                    }
                    AllBRResult = dbContext.OracleExcuteDatatable($"select distinct A.TEST_ITEM_ID,A.ENGLISH_NAME,A.CHINESE_NAME,A.QUANTITATIVE_RESULT,B.INPATIENT_ID,A.INSERT_TIME from  xhlis.his_inspection_result A left join xhlis.his_inspection_sample B on A.INSPECTION_ID=B.INSPECTION_ID where A.INSPECTION_ID in (select INSPECTION_ID from xhlis.his_inspection_sample where INPATIENT_ID in {InJZK.ToString()})");
                    log.Debug($"select distinct A.TEST_ITEM_ID,A.ENGLISH_NAME,A.CHINESE_NAME,A.QUANTITATIVE_RESULT,B.INPATIENT_ID,A.INSERT_TIME from  xhlis.his_inspection_result A left join xhlis.his_inspection_sample B on A.INSPECTION_ID=B.INSPECTION_ID where A.INSPECTION_ID in (select INSPECTION_ID from xhlis.his_inspection_sample where INPATIENT_ID in {InJZK.ToString()})");
                    dataTables.Add(AllBRResult);
                    AllBRResult = DataTableTool.DataTableMerge(dataTables[0], dataTables);
                    //将数据插入到临时表
                    //CommandCollection command = new CommandCollection();
                    //for (int i = 0; i < AllBRResult.Rows.Count; i++)
                    //{
                    //    command.Add(new Command($"INSERT INTO ResultTable VALUES('{AllBRResult.Rows[i]["TEST_ITEM_ID"]}','{AllBRResult.Rows[i]["ENGLISH_NAME"]}','{AllBRResult.Rows[i]["CHINESE_NAME"]}','{AllBRResult.Rows[i]["QUANTITATIVE_RESULT"]}','{AllBRResult.Rows[i]["INPATIENT_ID"]}','{AllBRResult.Rows[i]["INSERT_TIME"]}')"));
                    //}
                    //dbContext.ChangeContext(DBType.OracleResult).ExecuteTransaction(command);
                }
            }
            catch (Exception ex)
            {
                log.Error(ex, ex.Message);
                throw new Exception(ex.Message);
            }
            return AllBRResult;
        }

        /// <summary>
        /// 清空临时表
        /// </summary>
        private void TruncateResult()
        {
            string sql = "TRUNCATE TABLE ResultTable";
            dbContextResult.ExecuteSql(sql);
            log.Debug("临时表数据已清除");
        }


        #endregion

        public void Dispose()
        {
            log.Debug("OracleSourceDB资源被释放");
            AllBRResult = null;
            GC.SuppressFinalize(this);
        }
    }
}
