﻿using AnalyzeTask.DataSource;
using AnalyzeTask.SchedulingTasks;
using AnalyzeTask.Tool;
using Quartz;
using Serilog;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace AnalyzeTask.QuartzPack
{
    public class MyJobListener : JobListener
    {
        private ILogger log = Serilogs.logger;
        public override string Name => base.Name;
        /// <summary>
        /// 当任务执行前加载数据源
        /// </summary>
        /// <param name="context"></param>
        private static int JobIndex = 0;
        private static int WasIndex = 0;
        public int JobCount = BaseJob.jobDetails.Count;
        private NeedSynData needSynData = new NeedSynData();
        private OracleSourceDB oracleSourceDB = new OracleSourceDB();
        public override void JobToBeExecuted(IJobExecutionContext context)
        {
            lock (this)
            {
                try
                {
                    if (JobIndex == 0)
                    {
                        log.Information($"正在加载数据,当前索引{JobIndex}");
                        JobIndex++;
                        needSynData.GetNeedSynData();
                        var BR = NeedSynData.pregnancies.Select(n => n.AttendanceCard).ToArray();
                        oracleSourceDB.GetAllCheckResultByJZKOne(BR);
                        log.Information("数据加载完成");
                        log.Information($"{NeedSynData.DateTimeList.Count}|{NeedSynData.pregnancies.Count}|{NeedSynData.tableConfigs.Count}");
                        log.Debug($"日期数：{NeedSynData.DateTimeList.Count}|人数：{NeedSynData.pregnancies.Count}|表配置：{NeedSynData.tableConfigs.Count}|检查结果：{OracleSourceDB.AllBRResult.Rows.Count}");
                    }
                }
                catch (Exception ex)
                {
                    log.Error(ex, ex.Message);
                }

            }
        }

        public override void JobWasExecuted(IJobExecutionContext context, JobExecutionException jobException)
        {
            WasIndex++;
            if (WasIndex == JobCount)
            {
                WasIndex = 0;
                //释放所有公共资源
                needSynData.Dispose();
                oracleSourceDB.Dispose();
                log.Information("--------所有任务执行完成--------");
            }
        }
    }
}
